﻿/*
 * Extends chart control
 *
 * Author   : Mikhail Shiryaev
 * Created  : 2016
 * Modified : 2016
 *
 * Requires:
 * - jquery
 * - hummer
 * - utils.js
 * - chart.js
 */

/********** Default Phrases **********/

// Default phrases object
scada.chart.defaultPhrases = {
    ZoomIn: "Zoom In",
    ZoomOut: "Zoom Out",
    ResetZoom: "Reset Zoom"
}

/********** Time Selection **********/

// Time selection type
scada.chart.TimeSelection = function () {
    // Absolute start coordinate of the selection relative to the document
    this.absStartX = 0;
    // Absolute end coordinate of the selection relative to the document
    this.absEndX = 0;
    // Selection is in progress
    this.inProgress = false;
}

/********** Pinch Data **********/

// Pinch data type
scada.chart.PinchData = function (pinchEvent) {
    // Zoom scale
    this.scale = 1;
    // X-coordinate of the gesture center relative to the chart
    this.centerX = 0;
    // Y-coordinate of the gesture center relative to the chart
    this.centerY = 0;

    this._init(pinchEvent);
}

// Initialize the pinch data based on the specified event
scada.chart.PinchData.prototype._init = function (pinchEvent) {
    this.scale = pinchEvent.scale;
    this.centerX = pinchEvent.center.x;
    this.centerY = pinchEvent.center.y;
}

/********** Extension of Chart Control **********/

// Time selection set by a user
scada.chart.Chart.prototype._timeSelection = new scada.chart.TimeSelection();
// Time selection jQuery element
scada.chart.Chart.prototype._timeSelElem = null;
// Pinch information jQuery element
scada.chart.Chart.prototype._pinchInfoElem = null;
// Reset zoom button
scada.chart.Chart.prototype._resetZoomBtn = null;
// Localized phrases used by the chart
scada.chart.Chart.prototype.phrases = scada.chart.defaultPhrases;

// Create a time selection element if it doesn't exist
scada.chart.Chart.prototype._initTimeSelElem = function () {
    if (!this._timeSelElem) {
        this._timeSelElem = $("<div class='chart-time-sel hidden'></div>");
        this._canvasJqObj.after(this._timeSelElem);
    }
}

// Start time selection from the specified point
scada.chart.Chart.prototype._startTimeSelection = function (pageX, pageY) {
    var layout = this._chartLayout;
    layout.updateAbsCoordinates(this._canvasJqObj);

    pageX = Math.floor(pageX);
    pageY = Math.floor(pageY);

    if (layout.pointInPlotArea(pageX, pageY)) {
        this._timeSelection.absStartX = pageX;
        this._timeSelection.absEndX = pageX;
        this._timeSelection.inProgress = true;

        this._initTimeSelElem();
        this._timeSelElem        
            .css({
                "left": pageX - layout.absCanvasLeft,
                "top": layout.plotAreaTop,
                "width": 1,
                "height": layout.plotAreaHeight
            })
            .removeClass("hidden");

        return true;
    } else {
        this._cancelTimeSelection();
        return false;
    }
}

// Continue time selection at the specified point
scada.chart.Chart.prototype._continueTimeSelection = function (pageX, pageY) {
    var layout = this._chartLayout;

    pageX = Math.floor(pageX);
    pageY = Math.floor(pageY);

    if (this._timeSelection.inProgress && layout.pointInPlotArea(pageX, pageY)) {
        this._timeSelection.absEndX = pageX;
        var startX = this._timeSelection.absStartX;

        if (pageX >= startX) {
            this._timeSelElem.css({
                "left": startX - layout.absCanvasLeft,
                "width": pageX - startX + 1
            });
        } else {
            this._timeSelElem.css({
                "left": pageX - layout.absCanvasLeft,
                "width": startX - pageX + 1
            });
        }
    }
}

// Stop time selection at the specified point and apply the new scale
scada.chart.Chart.prototype._stopTimeSelection = function () {
    var timeSel = this._timeSelection;

    if (timeSel.inProgress) {
        this._cancelTimeSelection();

        if (timeSel.absStartX != timeSel.absEndX) {
            var startX = this._pageXToTrendX(timeSel.absStartX);
            var endX = this._pageXToTrendX(timeSel.absEndX);
            this.setRange(startX, endX);
        }

        this._showHideResetZoomBtn();
    }
}

// Cancel time selection
scada.chart.Chart.prototype._cancelTimeSelection = function () {
    this._timeSelection.inProgress = false;
    if (this._timeSelElem) {
        this._timeSelElem.addClass("hidden");
    }
}

// Create a pinch information element if it doesn't exist
scada.chart.Chart.prototype._initPinchInfoElem = function () {
    if (!this._pinchInfoElem) {
        this._pinchInfoElem = $("<div class='chart-trend-hint hidden'></div>");
        this._canvasJqObj.after(this._pinchInfoElem);
    }
}

// Start pinch zoom process
scada.chart.Chart.prototype._startPinchZoom = function (pinchData) {
    this._hintEnabled = false;
    this._continuePinchZoom(pinchData);
}

// Continue pinch zoom process
scada.chart.Chart.prototype._continuePinchZoom = function (pinchData) {
    var layout = this._chartLayout;
    layout.updateAbsCoordinates(this._canvasJqObj);

    this._initPinchInfoElem();
    var pinchInfoElem = this._pinchInfoElem;

    // construct zoom info
    var scaleStr = Math.round(pinchData.scale * 10) / 10.0;
    var infoText = (pinchData.scale >= 1 ? this.phrases.ZoomIn : this.phrases.ZoomOut) +
        " " + scaleStr + "x";

    // allow measuring the pinch info element
    pinchInfoElem
        .text(infoText)
        .css({
            "left": 0,
            "top": 0,
            "visibility": "hidden"
        })
        .removeClass("hidden");

    // display the pinch info element
    pinchInfoElem.css({
        "left": pinchData.centerX - layout.absCanvasLeft - pinchInfoElem.outerWidth() / 2,
        "top": pinchData.centerY - layout.absCanvasTop - layout.HINT_OFFSET - pinchInfoElem.outerHeight(),
        "visibility": ""
    });
}

// Stop pinch zoom process
scada.chart.Chart.prototype._stopPinchZoom = function (pinchData) {
    this._cancelPinchZoom();
    var timeRange = this._getTimeRange(pinchData);
    this.setRange(timeRange.startTime, timeRange.endTime);
    this._showHideResetZoomBtn();
}

// Cancel pinch zoom process
scada.chart.Chart.prototype._cancelPinchZoom = function () {
    this._hintEnabled = true;
    this._pinchInfoElem.addClass("hidden");
}

// Get time range according to the specified pinch data
scada.chart.Chart.prototype._getTimeRange = function (pinchData) {
    var centerTime = this._pageXToTrendX(pinchData.centerX);
    var rangeWidth = (this._maxX - this._minX) / pinchData.scale;
    var startTime = centerTime - rangeWidth / 2;
    var endTime = startTime + rangeWidth;

    if (startTime < this.timeRange.startTime) {
        var shift = this.timeRange.startTime - startTime;
        startTime = this.timeRange.startTime;
        endTime += shift;
    }

    if (endTime > this.timeRange.endTime) {
        endTime = this.timeRange.endTime;
    }

    var timeRange = new scada.chart.TimeRange();
    timeRange.startTime = startTime;
    timeRange.endTime = endTime;
    return timeRange;
}

// Create a reset zoom button if it doesn't exist
scada.chart.Chart.prototype._initResetZoomBtn = function () {
    if (!this._resetZoomBtn) {
        this._resetZoomBtn = $("<div class='chart-reset-zoom hidden' title='" + this.phrases.ResetZoom + "'>" +
            "<span class='glyphicon glyphicon-zoom-out'></span></div>");
        this._canvasJqObj.after(this._resetZoomBtn);

        var thisObj = this;
        this._resetZoomBtn.on("mousedown", function (event) {
            event.stopPropagation();
            $(this).addClass("hidden");
            thisObj.resetRange();
        });
    }
}

// Determine visibility of the reset zoom button
scada.chart.Chart.prototype._showHideResetZoomBtn = function () {
    this._initResetZoomBtn();

    if (this._zoomMode) {
        this._resetZoomBtn
            .css({
                "right": this._chartLayout.RIGHT_PADDING,
                "top": this._chartLayout.canvasTopBorder + this._chartLayout.plotAreaTop
            })
            .removeClass("hidden");
    } else {
        this._resetZoomBtn.addClass("hidden");
    }
}

// Bind events to allow scaling
scada.chart.Chart.prototype.bindScalingEvents = function () {
    if (this._canvasJqObj.length) {
        var thisObj = this;

        // events for desktop
        $(this._canvasJqObj.parent())
        .off(".scada.chart.scaling")
        .on("touchstart.scada.chart.scaling", function () {
            // switch off mouse selection for tablets
            $(this).off("mousedown.scada.chart.scaling");
        })
        .on("mousedown.scada.chart.scaling", function (event) {
            if (thisObj._startTimeSelection(event.pageX, event.pageY)) {
                event.preventDefault();
            }
        })
        .on("mousemove.scada.chart.scaling", function (event) {
            thisObj._continueTimeSelection(event.pageX, event.pageY);
        })
        .on("mouseup.scada.chart.scaling", function () {
            thisObj._stopTimeSelection();
        });

        // touch events
        var touchArea = this._canvasJqObj[0];
        var mc = new Hammer(touchArea);
        var mc = new Hammer.Manager(touchArea, {
            recognizers: [
                [Hammer.Pinch, { enable: true }]
            ]
        });

        mc
        .on("pinchstart", function (event) {
            thisObj._startPinchZoom(new scada.chart.PinchData(event));
        })
        .on("pinchmove", function (event) {
            thisObj._continuePinchZoom(new scada.chart.PinchData(event));
        })
        .on("pinchend", function (event) {
            thisObj._stopPinchZoom(new scada.chart.PinchData(event));
        })
        .on("pinchcancel", function () {
            thisObj._cancelPinchZoom();
        });
    }
}